from measured import Charge, Length, Mass, Time
from measured.natural import StoneyCharge, StoneyLength, StoneyMass, StoneyTime
from measured.physics import G, c, e, α, ℏ
from measured.si import Coulomb, Kilogram, Meter, Second


def test_unity() -> None:
    c.assert_approximates(1 * (StoneyLength / StoneyTime))
    G.assert_approximates(1 * (StoneyLength**3 / (StoneyMass * StoneyTime**2)))
    e.assert_approximates(1 * StoneyCharge)

    # TODO: this is failing to cancel terms fully
    # kₑ.assert_approximates(
    #     1 * ((AtomicLength**3 * AtomicMass) / (AtomicTime**2 * AtomicCharge**2))
    # )

    ℏ.assert_approximates(α**-1 * ((StoneyLength**2 * StoneyMass) / StoneyTime))


def test_length() -> None:
    assert StoneyLength.dimension is Length
    (1 * StoneyLength).assert_approximates(1.3807e-36 * Meter, within=2e-5)


def test_mass() -> None:
    assert StoneyMass.dimension is Mass
    (1 * StoneyMass).assert_approximates(1.8592e-9 * Kilogram, within=5e-6)


def test_time() -> None:
    assert StoneyTime.dimension is Time
    (1 * StoneyTime).assert_approximates(4.6054e-45 * Second, within=2e-5)


def test_charge() -> None:
    assert StoneyCharge.dimension is Charge
    (1 * StoneyCharge).assert_approximates(1.6022e-19 * Coulomb, within=2e-5)



from measured import Length, Mass, Temperature, Time
from measured.natural import PlanckLength, PlanckMass, PlanckTemperature, PlanckTime
from measured.physics import G, c, k, ℏ
from measured.si import Joule, Kelvin, Kilogram, Meter, Second


def test_unity() -> None:
    c.assert_approximates(1 * (PlanckLength / PlanckTime))
    ℏ.assert_approximates(1 * ((PlanckLength**2 * PlanckMass) / PlanckTime))
    G.assert_approximates(1 * (PlanckLength**3 / (PlanckMass * PlanckTime**2)))
    k.assert_approximates(
        1 * ((PlanckLength**2 * PlanckMass) / (PlanckTemperature * PlanckTime**2))
    )


def test_hbar() -> None:
    ℏ.assert_approximates(1.054571817e-34 * Joule * Second)


def test_length() -> None:
    assert PlanckLength.dimension is Length
    (1 * PlanckLength).assert_approximates(1.616255e-35 * Meter)


def test_mass() -> None:
    assert PlanckMass.dimension is Mass
    (1 * PlanckMass).assert_approximates(2.176434e-8 * Kilogram)


def test_time() -> None:
    assert PlanckTime.dimension is Time
    (1 * PlanckTime).assert_approximates(5.391247e-44 * Second)


def test_temperature() -> None:
    assert PlanckTemperature.dimension is Temperature
    (1 * PlanckTemperature).assert_approximates(1.416784e32 * Kelvin)



from measured import Charge, Length, Mass, Time
from measured.natural import AtomicCharge, AtomicLength, AtomicMass, AtomicTime
from measured.physics import c, e, mₑ, α, ℏ
from measured.si import Coulomb, Kilogram, Meter, Second


def test_unity() -> None:
    e.assert_approximates(1 * AtomicCharge)
    mₑ.assert_approximates(1 * AtomicMass)
    ℏ.assert_approximates(1 * ((AtomicLength**2 * AtomicMass) / AtomicTime))

    # TODO: this is failing to cancel terms fully
    # kₑ.assert_approximates(
    #     1 * ((AtomicLength**3 * AtomicMass) / (AtomicTime**2 * AtomicCharge**2))
    # )

    c.assert_approximates(α**-1 * (AtomicLength / AtomicTime))


def test_length() -> None:
    assert AtomicLength.dimension is Length
    (1 * AtomicLength).assert_approximates(5.292e-11 * Meter, within=5e-5)


def test_mass() -> None:
    assert AtomicMass.dimension is Mass
    (1 * AtomicMass).assert_approximates(9.109e-31 * Kilogram, within=5e-5)


def test_time() -> None:
    assert AtomicTime.dimension is Time
    (1 * AtomicTime).assert_approximates(2.419e-17 * Second, within=5e-5)


def test_charge() -> None:
    assert AtomicCharge.dimension is Charge
    (1 * AtomicCharge).assert_approximates(1.6022e-19 * Coulomb, within=2e-05)



from measured import Charge, Length, Mass, Time
from measured.natural import NaturalCharge, NaturalLength, NaturalMass, NaturalTime
from measured.physics import c, mₑ, ℏ
from measured.si import Coulomb, Kilogram, Meter, Second


def test_unity() -> None:
    c.assert_approximates(1 * (NaturalLength / NaturalTime))
    mₑ.assert_approximates(1 * NaturalMass)
    ℏ.assert_approximates(1 * ((NaturalLength**2 * NaturalMass) / NaturalTime))

    # TODO: this is failing to cancel terms fully
    # ε0.assert_approximates(
    #     1
    #     * ((NaturalTime**2 * NaturalCharge**2) / (NaturalMass * NaturalLength**3))
    # )


def test_length() -> None:
    assert NaturalLength.dimension is Length
    (1 * NaturalLength).assert_approximates(3.862e-13 * Meter, within=2e-4)


def test_mass() -> None:
    assert NaturalMass.dimension is Mass
    (1 * NaturalMass).assert_approximates(9.109e-31 * Kilogram, within=5e-5)


def test_time() -> None:
    assert NaturalTime.dimension is Time
    (1 * NaturalTime).assert_approximates(1.288e-21 * Second, within=7e-5)


def test_charge() -> None:
    assert NaturalCharge.dimension is Charge
    (1 * NaturalCharge).assert_approximates(5.291e-19 * Coulomb, within=4e-5)



from typing import Union

SUPERSCRIPTS = {
    "-": "⁻",
    ".": ".",  # There does not seem to be a superscript '.' in Unicode yet
    **{
        str(i): v
        for i, v in enumerate(["⁰", "¹", "²", "³", "⁴", "⁵", "⁶", "⁷", "⁸", "⁹"])
    },
}
DIGITS = {v: k for k, v in SUPERSCRIPTS.items()}


def superscript(exponent: Union[int, float]) -> str:
    """Given a signed integer exponent, returns the Unicode superscript string for it

    Examples:

        >>> f"x{superscript(123)}"
        'x¹²³'
        >>> f"x{superscript(0)}"
        'x⁰'
        >>> f"x{superscript(-1)}"
        'x⁻¹'
        >>> f"x{superscript(1)}"
        'x'
    """
    if exponent == 1:
        return ""

    return "".join(SUPERSCRIPTS[c] for c in str(exponent))


def from_superscript(string: str) -> int:
    return int("".join(DIGITS[c] for c in string))
